

#include "pin.H"

#include <iostream>
#include <fstream>
#include <ctime>
#include <string>
#include <cstdio>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <pthread.h>

#if defined(TARGET_MAC)
#include <sys/syscall.h>
#elif !defined(TARGET_WINDOWS)
#include <syscall.h>
#endif

//Includes for MySQL db store of output
#include <mysql.h>

using namespace std;

//These are the performance models now only considering the model3LevelCache
//#include "performance_model_3LevelCache.h"

/* ================================================================== */
// Global variables 
/* ================================================================== */

UINT64 insCount = 0;        //number of dynamically executed instructions
UINT64 bblCount = 0;        //number of dynamically executed basic blocks
UINT64 threadCount = 0;     //total number of threads, including main thread
UINT64	mallocCount = 0;	//Total number of malloc
UINT64  freeCount = 0;		//Total number of free
UINT64	barrierCount = 0;	//Total of barriers detected
UINT64	memoryReadCount = 0; //Total of memory reads
UINT64  memoryWriteCount = 0; //Total of memory writes

UINT64	writeSyscallCount = 0; //Total bytes writed 
UINT64  readSyscallCount = 0; //Total bytes readed
UINT64  isWriteFDValue = 0; //Value of descriptor
UINT64  isReadFDValue = 0; //Value of descriptor
UINT64  recvSyscallCount = 0; //Total bytes received from network
UINT64	sendSSyscallCount = 0; //Total bytes sended to network

UINT64 arrayFDOpenFiles[500]; //Array of FD descriptors of open files

//UINT64 test=-1;



//Declare variables for DB connection to store output of instrumentation
MYSQL *conn;
MYSQL_RES *res;
MYSQL_ROW row;



char *worker = (char *) malloc(100);
char *workname = (char *) malloc(100);


//Mutex for MySQL inserts
pthread_mutex_t lockmysql;


//Variables for instrument syscalls
int isWrite=0;
int isRead=0;
int isRecv=0;
int isSend=0;
int isOpen=0;
int isClose=0;
//Directory to instrument syscalls read/write. If file is contanied on this
//directory we put bytes on vars, if outside ignore
char *dirForSyscalls= (char *) malloc(500);


//Vars to control instrumentation
//we can choose if we want or not to instrument based on fileconf
//if no conf var or value != from 1, not instrument
//ins, bbl and thread are instrumented always

int ifMalloc = 0;
int ifFree = 0;
int ifBarrier = 0;
int ifMemoryRead = 0;
int ifMemoryWrite = 0;
//Syscall
int ifWrite = 0;
int ifRead = 0;
int ifRecv = 0;
int ifSend = 0;
int ifOpen = 0; //To an adecuate management of read/write we need this, don't remove it
int ifClose = 0; //To an adecuate management of read/write we need this, don't remove it


/* ===================================================================== */
/* Names of malloc and free */
/* ===================================================================== */
#define MALLOC "malloc"
#define FREE "free"
#define BARRIER "pthread_cond_wait"
PIN_LOCK lock;

std::ostream * out = &cerr;

/* ===================================================================== */
// Command line switches
/* ===================================================================== */
KNOB<string> KnobOutputFile(KNOB_MODE_WRITEONCE,  "pintool",
    "o", "", "File for Apache Spark instrumentation output");

KNOB<BOOL>   KnobCount(KNOB_MODE_WRITEONCE,  "pintool",
    "count", "1", "This tool instruments Apache Spark workloads");


/**
 * Returns 1 if char b is contained on char a, else returns 0
 * File comparison begins from the [0] index of the char value
 * for example
 * 
 * a='/servers/spark/data/file1.data'
 * b='/servers/spark'
 * 
 * returns 1
 * 
 * a='/servers/spark/data/file1.data'
 * b='/mydata/servers/spark'
 * 
 * return 0
 * 
 **/
int contains(char * a, char * b){
	
	int size=strlen(b);
	//printf("Size:%d\n",size);
	//printf("A.%s\n",a);
	//printf("B.%s\n",b);
	if (strncmp(a,b,size)==0){
		//a and b are equal until b size
		//printf("1.%s\n",a);
		return(1);
			
	} else {
		//printf("2.%s\n",b);
		return(0);
		
	}
	
	
}

//Put this FD descriptor on OpenFiles array
VOID addToOpenFiles(ADDRINT ret)

{

	*out <<  "Adding file: " << ret << endl;
	int i;
	for (i=0; i<499;i++){
		if (arrayFDOpenFiles[i]==987654321){
			//This value is dummy, empty space
			arrayFDOpenFiles[i]=ret;
			
			break;
		}
		
	
		
	}
	
	//printf("Insert on vector, Value for i %d, Vaalue for ret %ld\n",i,ret);
	
}

//Put this FD descriptor on OpenFiles array
VOID removeToOpenFiles(ADDRINT ret)

{
	
	*out <<  "Removing file: " << ret << endl;
	int i;
	for (i=0; i<499;i++){
		if (arrayFDOpenFiles[i]==ret){
			//We need to remove this value
			arrayFDOpenFiles[i]=987654321;
			break;
		}
		
		
	}
	
	
	
}

/**
 * Returns 1 if ret is in openfiles array
 * else returns 0
 * 
 * 
 */
int isFDinOpenFiles(ADDRINT ret)
{
	
	*out <<  "Search for file: " << ret << endl;
	int i;
	for (i=0; i<499;i++){
		//printf("Value on array position %d is %d\n",i,arrayFDOpenFiles[i]);
		if (arrayFDOpenFiles[i]==ret){
			
			//FD is on the array
			
			*out <<  "We find and interesting file " << i << "," << ret<< endl;
			return (1);
		}
		
	}
	
	return(0);
		
}





VOID partialLog(){
	
	pthread_mutex_lock(&lockmysql);

    *out <<  "===============================================" << endl;
    *out <<  "pinSpark analysis results: " << endl;
    *out <<  "Number of instructions: " << insCount  << endl;
    *out <<  "Number of basic blocks: " << bblCount  << endl;
    *out <<  "Number of threads: " << threadCount  << endl;
    *out <<  "Number of malloc: " << mallocCount  << endl;
    *out <<  "Number of free: " << freeCount << endl;
    *out <<  "Number of Memory Writes: " << memoryWriteCount << endl;
    *out <<  "Number of Memory Reads: " << memoryReadCount << endl;
    *out <<  "Number of barrier: " << barrierCount << endl;
    *out <<  "Bytes writed to disk: " << writeSyscallCount << endl;
    *out <<  "Bytes readed from disk: " << readSyscallCount << endl;
    *out <<  "Bytes received from network: " << recvSyscallCount << endl;
    *out <<  "Bytes sended to the network: " << sendSSyscallCount << endl;
    *out <<  "===============================================" << endl;
    
    //Partial insert of values on MySQL database as instrumentation runs
    
    //worker, number of the worker, for example 0
    //workname, filename of output, we use it as work name
    
   
   
    
   char query1[800] = {0};


    sprintf(query1,"insert into event(worker,workname,date, insCount,bblCount,threadCount,mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount,writeSyscallCount,readSyscallCount,recvSyscallCount,sendSSyscallCount) values (%s,'%s',NOW(),%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ")" ,worker,workname,insCount,bblCount,threadCount, mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount, writeSyscallCount, readSyscallCount, recvSyscallCount, sendSSyscallCount);
    //*out << query1 << endl;
 
  
 
   

	int err;
   
    err=mysql_real_query(conn, query1,strlen(query1));

    if (err!=0){
		*out << "Error on query: " << err << endl;
		*out << query1 << endl; 
		
	}
	
	pthread_mutex_unlock(&lockmysql);
    


   
    

}





/* ===================================================================== */
// Utilities
/* ===================================================================== */

/*!
 *  Print out help message.
 */
INT32 Usage()
{
    cerr << "This tool instruments Apache Spark workloads for performance evaluation " << endl;

    cerr << KNOB_BASE::StringKnobSummary() << endl;

    return -1;
}

/* ===================================================================== */
// Analysis routines
/* ===================================================================== */

/*!
 * Increase counter of the executed basic blocks and instructions.
 * This function is called for every basic block when it is about to be executed.
 * @param[in]   numInstInBbl    number of instructions in the basic block
 * @note use atomic operations for multi-threaded applications
 */
VOID CountBbl(UINT32 numInstInBbl)
{
    bblCount++;
    insCount += numInstInBbl;
	
    if (bblCount % 1000000000 ==0)
	{
	//Get partial statistics

 	partialLog();	
	}
}

/* ===================================================================== */
// Instrumentation callbacks
/* ===================================================================== */

/*!
 * Insert call to the CountBbl() analysis routine before every basic block 
 * of the trace.
 * This function is called every time a new trace is encountered.
 * @param[in]   trace    trace to be instrumented
 * @param[in]   v        value specified by the tool in the TRACE_AddInstrumentFunction
 *                       function call
 */
VOID Trace(TRACE trace, VOID *v)
{
    // Visit every basic block in the trace
    for (BBL bbl = TRACE_BblHead(trace); BBL_Valid(bbl); bbl = BBL_Next(bbl))
    {
        // Insert a call to CountBbl() before every basic bloc, passing the number of instructions
        BBL_InsertCall(bbl, IPOINT_BEFORE, (AFUNPTR)CountBbl, IARG_UINT32, BBL_NumIns(bbl), IARG_END);
    }
}

/*!
 * Increase counter of threads in the application.
 * This function is called for every thread created by the application when it is
 * about to start running (including the root thread).
 * @param[in]   threadIndex     ID assigned by PIN to the new thread
 * @param[in]   ctxt            initial register state for the new thread
 * @param[in]   flags           thread creation flags (OS specific)
 * @param[in]   v               value specified by the tool in the 
 *                              PIN_AddThreadStartFunction function call
 */
VOID ThreadStart(THREADID threadIndex, CONTEXT *ctxt, INT32 flags, VOID *v)
{
    threadCount++;
}






VOID Fini(INT32 code, VOID *v)
{

	pthread_mutex_lock(&lockmysql);

    *out <<  "===============================================" << endl;
    *out <<  "pinSpark analysis results: " << endl;
    *out <<  "Number of instructions: " << insCount  << endl;
    *out <<  "Number of basic blocks: " << bblCount  << endl;
    *out <<  "Number of threads: " << threadCount  << endl;
    *out <<  "Number of malloc: " << mallocCount  << endl;
    *out <<  "Number of free: " << freeCount << endl;
    *out <<  "Number of Memory Writes: " << memoryWriteCount << endl;
    *out <<  "Number of Memory Reads: " << memoryReadCount << endl;
    *out <<  "Number of barrier: " << barrierCount << endl;
    *out <<  "Bytes writed to disk: " << writeSyscallCount << endl;
    *out <<  "Bytes readed from disk: " << readSyscallCount << endl;
    *out <<  "Bytes received from network: " << recvSyscallCount << endl;
    *out <<  "Bytes sended to the network: " << sendSSyscallCount << endl;
    *out <<  "===============================================" << endl;

      //printPerformanceStats(); 

    //Partial insert of values on MySQL database as instrumentation runs
    
    //worker, number of the worker, for example 0
    //workname, filename of output, we use it as work name
    


    //Old values
   // sprintf(query1,"insert into event(worker,workname,date, insCount,bblCount,threadCount,mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount) values (%s,'%s',NOW(),%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ")" ,worker,workname,insCount,bblCount,threadCount, mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount);
    
    
     
    char query1[800] = {0};


    sprintf(query1,"insert into event(worker,workname,date, insCount,bblCount,threadCount,mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount,writeSyscallCount,readSyscallCount,recvSyscallCount,sendSSyscallCount) values (%s,'%s',NOW(),%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ",%" PRIu64 ")" ,worker,workname,insCount,bblCount,threadCount, mallocCount, freeCount, barrierCount, memoryReadCount, memoryWriteCount, writeSyscallCount, readSyscallCount, recvSyscallCount, sendSSyscallCount);
    //*out << query1 << endl;
 
	int err;
   
    err=mysql_real_query(conn, query1,strlen(query1));

    if (err!=0){
		*out << "Error on query: " << err << endl;
		*out << query1 << endl; 
		
	}
 


    mysql_close(conn);
    
    pthread_mutex_unlock(&lockmysql);
}


/**
 * 
 * This funcion is executed on each malloc
 * 
 */
VOID mallocCalculation(ADDRINT size, THREADID threadid)
{

	PIN_GetLock(&lock, threadid+1);
	mallocCount++;
	PIN_ReleaseLock(&lock);

}

VOID freeCalculation(ADDRINT size, THREADID threadid)
{

	PIN_GetLock(&lock, threadid+1);
	freeCount++;
	PIN_ReleaseLock(&lock);

}

VOID barrierCalculation(ADDRINT size, THREADID threadid)
{

	PIN_GetLock(&lock, threadid+1);
	barrierCount++;
	PIN_ReleaseLock(&lock);

}

VOID memoryReadCalculation(VOID * ip,  UINT32 size)
{


	memoryReadCount++;


}

VOID memoryWriteCalculation(VOID * ip,  UINT32 size)
{


	memoryWriteCount++;


}

// Print syscall number and arguments
VOID SysBefore(ADDRINT ip, ADDRINT num, ADDRINT arg0, ADDRINT arg1, ADDRINT arg2, ADDRINT arg3, ADDRINT arg4, ADDRINT arg5)
{
#if defined(TARGET_LINUX) && defined(TARGET_IA32) 
    // On ia32 Linux, there are only 5 registers for passing system call arguments, 
    // but mmap needs 6. For mmap on ia32, the first argument to the system call 
    // is a pointer to an array of the 6 arguments
    if (num == SYS_mmap)
    {
        ADDRINT * mmapArgs = reinterpret_cast<ADDRINT *>(arg0);
        arg0 = mmapArgs[0];
        arg1 = mmapArgs[1];
        arg2 = mmapArgs[2];
        arg3 = mmapArgs[3];
        arg4 = mmapArgs[4];
        arg5 = mmapArgs[5];
    }
#endif

 
	if ((long)num == 1){
		//Write syscall
		//Test if we instrument
		if (ifWrite==1){
		
		//Before instrument, check if file is in the array
		
		*out << "Write number: " << arg0 << endl;
		if (isFDinOpenFiles(arg0)==1){
		
		*out << "Interesting File" << endl;
		//Ok, we're on 	a file of interest
		isWrite = 1;
		isWriteFDValue = arg0;
		}
		}
		
	} 
	

	if ((long)num == 0){
		//Read syscall
		//Test if we instrument
		if (ifRead==1){
		*out << "Read syscall on FD " << arg0 << endl;
		//Before instrument, check if file is in the array
		if (isFDinOpenFiles(arg0)==1){
		//Ok, we're on 	a file of interest
		isRead = 1;
		isReadFDValue = arg0;
		*out << "Read syscall on FD is on interesting file " << arg0 << endl;
		}
		}
		
	} 
	

	if ((long)num == 45){
		//Recv syscall
		//Test if we instrument
		if (ifRecv==1){
		*out << "Recv syscall received: " << arg0 << endl;
		isRecv = 1;
		}
	} 
	
	if ((long)num == 47){
		//Recvmsg syscall
		//Test if we instrument
		if (ifRecv==1){
		*out << "Recvmsg syscall received: " << arg0 << endl;
		isRecv = 1;
		}
	} 
	

	if ((long)num == 44){
		//Send syscall
		//Test if we instrument
		if (ifSend==1){
		*out << "Send syscall received: " << arg0 << endl;
		isSend = 1;
		}
		
	}
	
		if ((long)num == 46){
		//Sendmsg syscall
		//Test if we instrument
		if (ifSend==1){
		*out << "Sendmsg syscall received: " << arg0 << endl;
		isSend = 1;
		}
		
	}
	
	//open
	if ((long)num == 2){
		//Open syscall
		//Test if we instrument
		if (ifOpen==1){
		
		//arg0 store memory address of first parameter, name of file to open
    	char * address = (char *) arg0;
				
		//Comparefile from syscall with conf value
		
		//*out << "Open: File to compare 1: " << address<< endl;
		//*out << "Open: File to compare 2: " << dirForSyscalls << endl;
		if (contains(address,dirForSyscalls)==1)
		{
			//we must instrument 
			isOpen=1;
			*out << "Open: File is interesting to instrument: " << address<< endl;
		 
		}
		}

			
	}
	
	
	
	
	if ((long)num == 3){
		//Close syscall
		//Test if we instrument
		if (ifClose ==1){
			
		
		
			*out << "Close syscall requested " << endl;
			isClose=1;
			
		}
	}

}




VOID SysAfter(ADDRINT ret)
{
	
	//Test if syscall() returns -1 error
	//UINT64 vars don't store negative values
	
	if (isWrite == 1 || isRead == 1 || isRecv == 1 || isSend == 1 || isOpen == 1|| isClose == 1){
	if (ret == 18446744073709551615){
		
		*out << "Syscall error, return value: " << ret << endl;
		//Syscall is returning an error
		//No instrumentation
		isWrite = 0;
		isRead = 0;
		isRecv = 0;
		isSend = 0;
		isOpen = 0;
		isClose = 0;
		
		}
	}
 
    if (isWrite == 1){
		
		
		*out << "Add to write value: " << ret<< endl;
		writeSyscallCount+= ret;
		isWrite = 0;
		
	} 
	if (isRead == 1){
		*out << "Add to read value: " << ret<< endl;
		readSyscallCount+= ret;
		isRead = 0;
	}
	if (isRecv == 1){
		*out << "Add to isrecv value: " << ret<< endl;
		recvSyscallCount+= ret;
		isRecv = 0;
	} 
	if  (isSend == 1){
		*out << "Add to issend value: " << ret<< endl;
		sendSSyscallCount+= ret;
		isSend = 0;
	}
	
	if  (isOpen == 1){
		//Return value is the file descriptor
		//We need take this file descriptor in account until a close syscall closes file
		//file descriptors value are reused
		addToOpenFiles(ret);
		*out << "Opening file FD: " << ret << endl;
		isOpen = 0;
	}
	
	if (isClose == 1){
		//Closing File, we need to remove from array of interesting files
		//only remove if in this
		if (isFDinOpenFiles(ret)==1){
			removeToOpenFiles(ret);
			*out << "Closing file FD: " << ret << endl;
			
		}
		isClose = 0;
		
	}
	
	
   
    
}

VOID SyscallEntry(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v)
{
    SysBefore(PIN_GetContextReg(ctxt, REG_INST_PTR),
        PIN_GetSyscallNumber(ctxt, std),
        PIN_GetSyscallArgument(ctxt, std, 0),
        PIN_GetSyscallArgument(ctxt, std, 1),
        PIN_GetSyscallArgument(ctxt, std, 2),
        PIN_GetSyscallArgument(ctxt, std, 3),
        PIN_GetSyscallArgument(ctxt, std, 4),
        PIN_GetSyscallArgument(ctxt, std, 5));
}

VOID SyscallExit(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v)
{
    SysAfter(PIN_GetSyscallReturn(ctxt, std));
}


LOCALFUN VOID Image(IMG img, VOID *v)
{
    // Instrument the malloc() and free() functions.  Print the input argument
    // of each malloc() or free(), and the return value of malloc().
    //
    //  Find the malloc() function.
    RTN mallocRtn = RTN_FindByName(img, MALLOC);
    if (RTN_Valid(mallocRtn))
    {
		//Test if we instrument
		if (ifMalloc == 1){
        RTN_Open(mallocRtn);
        // Instrument malloc() to count operations 
        RTN_InsertCall(mallocRtn, IPOINT_BEFORE, (AFUNPTR)mallocCalculation,
                       IARG_ADDRINT, MALLOC,
                       IARG_FUNCARG_ENTRYPOINT_VALUE, 0, IARG_THREAD_ID,
                       IARG_END);
        RTN_Close(mallocRtn);}
    }

    // Find the free() function.
    RTN freeRtn = RTN_FindByName(img, FREE);
    if (RTN_Valid(freeRtn))
    {
		//Test if we instrument
		if (ifFree == 1){
        RTN_Open(freeRtn);
        // Instrument free() to count operations 
        RTN_InsertCall(freeRtn, IPOINT_BEFORE, (AFUNPTR)freeCalculation,
                       IARG_ADDRINT, FREE,
                       IARG_FUNCARG_ENTRYPOINT_VALUE, 0, IARG_THREAD_ID,
                       IARG_END);
        RTN_Close(freeRtn);}
    }
    
    //Search for barriers
    
    RTN barrierRtn = RTN_FindByName(img, BARRIER);
    if (RTN_Valid(barrierRtn)){
		//Test if we instrument
		if (ifBarrier==1){
		 RTN_Open(barrierRtn);
		RTN_InsertCall(barrierRtn, IPOINT_BEFORE, (AFUNPTR)barrierCalculation, IARG_ADDRINT, BARRIER,
                       IARG_FUNCARG_ENTRYPOINT_VALUE, 0, IARG_THREAD_ID,
                       IARG_END);
		RTN_Close(barrierRtn);}
		
	}
}


VOID Instruction(INS ins, VOID *v){
	 if (INS_IsMemoryRead(ins))
    {
		//Test if we instrument
		if (ifMemoryRead==1){
	INS_InsertPredicatedCall(ins, IPOINT_BEFORE, (AFUNPTR) memoryReadCalculation,
	IARG_INST_PTR,IARG_MEMORYWRITE_SIZE, IARG_END);}
	}
	
	if (INS_IsMemoryWrite(ins))
    {
		//Test if we instrument
		if (ifMemoryWrite==1){
		INS_InsertPredicatedCall(ins, IPOINT_BEFORE,(AFUNPTR) 
		memoryWriteCalculation,IARG_INST_PTR, IARG_MEMORYWRITE_SIZE, IARG_END);}
		
	}
	
	//Search for syscall
	  // For O/S's (Mac) that don't support PIN_AddSyscallEntryFunction(),
    // instrument the system call instruction.

    if (INS_IsSyscall(ins) && INS_HasFallThrough(ins))
    {
        // Arguments and syscall number is only available before
        INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(SysBefore),
                       IARG_INST_PTR, IARG_SYSCALL_NUMBER,
                       IARG_SYSARG_VALUE, 0, IARG_SYSARG_VALUE, 1,
                       IARG_SYSARG_VALUE, 2, IARG_SYSARG_VALUE, 3,
                       IARG_SYSARG_VALUE, 4, IARG_SYSARG_VALUE, 5,
                       IARG_END);

        // return value only available after
        INS_InsertCall(ins, IPOINT_AFTER, AFUNPTR(SysAfter),
                       IARG_SYSRET_VALUE,
                       IARG_END);
    }
	
	
}

int split(char *string, char *delim, char *tokens[])
{
    int count = 0;
    char *token;
    char *stringp;
 
    stringp = string;
    while (stringp != NULL) {
        token = strsep(&stringp, delim);
        tokens[count] = token;
        count++;
    }
    return count;
}

char *trimwhitespace(char *str)
{
  char *end;

  // Trim leading space
  while(isspace(*str)) str++;

  if(*str == 0)  // All spaces?
    return str;

  // Trim trailing space
  end = str + strlen(str) - 1;
  while(end > str && isspace(*end)) end--;

  // Write new null terminator
  *(end+1) = 0;

  return str;
}


int main(int argc, char *argv[])
{

char *server= (char *) malloc(100);
char *user= (char *) malloc(100);
char *password= (char *) malloc(100);
char *database= (char *) malloc(100);

	//Initialize array for FD descriptors on Syscall
	int i;
	for (i=0;i<499;i++)
	{
		arrayFDOpenFiles[i]=987654321; //We're sure that no FD are going to be used
	}
    
    //Read values from file conf
    //We set malloc on values to strcpy
   
    
 
    static const char filename[] = "/etc/pinSpark/pinSpark.conf";
    FILE *file = fopen ( filename, "r" );
    char * tokens[2];
        
      if ( file != NULL )
        {
        int count;
        char linea[128];
         while ( fgets ( linea, 128, file ) != NULL ) /* read a line */
               {
               count=split(linea,"=",tokens);
               //trim trash from vars
               tokens[0] = trimwhitespace(tokens[0]);
               tokens[1] = trimwhitespace(tokens[1]);
               
               //*out << "##" << endl;
         
               if (strcmp(tokens[0],"mysqlServer")==0)
               {
				   strcpy(server,tokens[1]);

			   }
			   
			   
			   if (strcmp(tokens[0],"mysqlDatabase")==0)
               {
			   strcpy(database,tokens[1]);

		       }
			   if (strcmp(tokens[0],"mysqlUser")==0)
			   {
				 strcpy(user,tokens[1]);
				
			   }
				 if (strcmp(tokens[0],"mysqlPassword")==0)
			   {
				strcpy(password,tokens[1]);
				
			   }
				 if (strcmp(tokens[0],"mysqlWorkerID")==0)
			   {
				strcpy(worker,tokens[1]);
				
			   }
			   
			   if (strcmp(tokens[0],"dirForSyscalls")==0)
			   {
				strcpy(dirForSyscalls,tokens[1]);   
			   }
			   			    			   
			    if (strcmp(tokens[0],"ifMalloc")==0 )
			   {
				 
				   ifMalloc = atoi(tokens[1]);
				  
				   
			   } 
			   
			    if (strcmp(tokens[0],"ifFree")==0 )
			   {
				  
				  ifFree=atoi(tokens[1]); 
				  
			   } 
			   
			    if (strcmp(tokens[0],"ifBarrier")==0 )
			   {
				   
				ifBarrier=atoi(tokens[1]);  
				
			   }
			   
			    if (strcmp(tokens[0],"ifMemoryRead")==0)
			   {
				
				ifMemoryRead=atoi(tokens[1]);   
			
			   } 
			   
			   if (strcmp(tokens[0],"ifMemoryWrite")==0)
			   {
			
				ifMemoryWrite=atoi(tokens[1]);
			
			   } 
			   
			    if (strcmp(tokens[0],"ifWrite")==0)
			   {
				    
				ifWrite=atoi(tokens[1]);  
			
			   } 
			   
			    if (strcmp(tokens[0],"ifRead")==0)
			   {
			
				ifRead=atoi(tokens[1]);   
				
			   } 
			   
			    if (strcmp(tokens[0],"ifRecv")==0)
			   {
				
				   	ifRecv=atoi(tokens[1]); 
			
			   } 
			   
			    if (strcmp(tokens[0],"ifSend")==0)
			   {
			
				ifSend=atoi(tokens[1]);  
			
			   } 
			   
			    if (strcmp(tokens[0],"ifOpen")==0)
			   {
		
				ifOpen=atoi(tokens[1]);  
			
			   } 
			   
			   if (strcmp(tokens[0],"ifClose")==0)
			   {
				   
				ifClose=atoi(tokens[1]);  
			
			   } 
	       }
        fclose ( file );
        } else
        
        {
			//Error on conf file access
		*out <<  "===============================================" << endl;
        *out <<  " Error reading /etc/pinSpark/pinSpark.conf     " << endl;
        *out <<  "===============================================" << endl;
        exit(1);
		}
    
	 conn = mysql_init(NULL);
	 
    
    //Connect to mysql 
    if (!mysql_real_connect(conn,server,user,password,database,0,NULL,0))
	{
	//Error on DB connnection
        *out <<  "===============================================" << endl;
        *out <<  " Error on Mysql DB connection, check conf      " << endl;
        *out <<  "===============================================" << endl;
        exit(1);
	}
	mysql_autocommit(conn, 1);
    
    
    
   

    if( PIN_Init(argc,argv) )
    {
        return Usage();
    }
    // Initialize the pin lock
    PIN_InitLock(&lock);
    
    PIN_InitSymbols();
    string fileName = KnobOutputFile.Value();
	
    //Output filename
    if (!fileName.empty()) { 
		
		out = new std::ofstream(fileName.c_str());
		//Setting workname to filename. We use this value for DB store of works
		strcpy(workname,fileName.c_str());
		*out << "Salida: " << workname << endl;
		}

        // Register function to be called to instrument traces
        //TRACE_AddInstrumentFunction(Trace, 0);

        // Register function to be called for every thread before it starts running
        //PIN_AddThreadStartFunction(ThreadStart, 0);


	// Register Image to be called to instrument functions.
    IMG_AddInstrumentFunction(Image, 0);
	TRACE_AddInstrumentFunction(Trace, 0);
	PIN_AddThreadStartFunction(ThreadStart, 0);
	INS_AddInstrumentFunction(Instruction, 0);
    
   
    PIN_AddSyscallEntryFunction(SyscallEntry, 0);
    PIN_AddSyscallExitFunction(SyscallExit, 0);

	// Register function to be called when the application exits
    PIN_AddFiniFunction(Fini, 0);

    
    cerr <<  "===============================================" << endl;
    cerr <<  "This application is instrumented by pinSpark" << endl;
    //err << "Valor negativo: " << test << endl;
    if (!KnobOutputFile.Value().empty()) 
    {
        cerr << "See file " << KnobOutputFile.Value() << " for analysis results" << endl;
    }
    cerr <<  "===============================================" << endl;

    // Start the program, never returns
    PIN_StartProgram();
    
    return 0;
}

