/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.spark.launcher

import java.io.File
import java.util.{HashMap => JHashMap, List => JList, Map => JMap}

import scala.collection.JavaConversions._

import org.apache.spark.deploy.Command

/** Imports for pinSpark **/
import com.typesafe.config.{ Config, ConfigFactory }
import scala.util.Random

/**
 * This class is used by CommandUtils. It uses some package-private APIs in SparkLauncher, and since
 * Java doesn't have a feature similar to `private[spark]`, and we don't want that class to be
 * public, needs to live in the same package as the rest of the library.
 */
private[spark] class WorkerCommandBuilder(sparkHome: String, memoryMb: Int, command: Command)
    extends AbstractCommandBuilder {

  childEnv.putAll(command.environment)
  childEnv.put(CommandBuilderUtils.ENV_SPARK_HOME, sparkHome)

  override def buildCommand(env: JMap[String, String]): JList[String] = {
    val cmd = buildJavaCommand(command.classPathEntries.mkString(File.pathSeparator))
    cmd.add(s"-Xms${memoryMb}M")
    cmd.add(s"-Xmx${memoryMb}M")
    command.javaOpts.foreach(cmd.add)
    addPermGenSizeOpt(cmd)
    addOptionString(cmd, getenv("SPARK_JAVA_OPTS"))
    val configPin = ConfigFactory.parseFile(new File("/etc/pinSpark/pinSpark.conf"))
    val cmd2: List[String] = List(configPin.getString("pinCommand"))
    val cmd21 = List.concat(cmd2,List(configPin.getString("pinToolSwitch")))
    val cmd22 = List.concat(cmd21,List(configPin.getString("pinTool")))
    val cmd23 = List.concat(cmd22,List(configPin.getString("pinOutSwitch")))
    //Generate unique filename, prefix from config file, path fixed on /tmp
    val logfile = randomFileName(configPin.getString("pinOutLogFile"),"log",configPin.getString("pinComputerName"),10,5) 
    val cmd24 = List.concat(cmd23,List(logfile))
    val cmd25 = List.concat(cmd24,List(configPin.getString("pinAttachSwitch")))
    println(cmd25)
    val cmd3 = List.concat(cmd25,cmd)
    println(cmd3)
    cmd3
  }


  def randomFileName(prefix: String = "", suffix: String = "", computer: String = "", maxTries: Int = 10, nameSize: Int = 10) = {
    val alphabet = ('a' to 'z') ++ ('A' to 'Z') ++ ('0' to '9') ++ ("_")
    def generateName = (1 to nameSize).map(_ => alphabet(Random.nextInt(alphabet.size))).mkString
    val name1 = "/tmp/" + prefix + "_" + computer +"_" + generateName + "."+suffix
    name1
  }

  def buildCommand(): JList[String] = buildCommand(new JHashMap[String, String]())

}
